# ------- generated by typeit -------
from typing import Any, NamedTuple, Optional, Sequence
# from typeit import TypeConstructor


class Data(NamedTuple):
    Token: str
    TTL: int
    Firstname: str
    Lastname: str
    Company: str
    Email: str
    Language: str
    Role: int
    Privacy: int
    PrivacyDate: str
    AvatarTimestamp: str
    Legal: Sequence


class Token(NamedTuple):
    Status: int
    ErrMsg: str
    ts: str
    Data: Data


# mk_main, serialize_main =  Main


"""Provide an abstract base class for easier requests."""
import abc
from typing import Optional as Mapping, Tuple, Mapping

from pyinim.cloud.exceptions import MalformedResponseError
from pyinim.cloud.resolver import CloudResolver
from pyinim.cloud.types.token import Token
from pyinim.cloud.types.devices import Devices

class InimAPI(abc.ABC):

    """Provide an idiomatic API for making calls to Inim's API."""

    def __init__(
        self,
        requester: str,
        *,
        resolver: CloudResolver,
    ) -> None:
        self.requester = requester
        self.resolver = resolver
        # self.rate_limit: Opt[RateLimit] = None

    @abc.abstractmethod
    async def _request(
        self, method: str, url: str, headers: Mapping[str, str], body: bytes = b""
    ) -> Tuple[int, Mapping[str, str], bytes]:
        """Make an HTTP request."""

    @abc.abstractmethod
    async def sleep(self, seconds: float) -> None:
        """Sleep for the specified number of seconds."""

    @abc.abstractmethod
    async def token(self) -> str:
        """Get the token string."""

    async def get_token(self) -> Tuple[int, Mapping[str, str], Token]:
        status, headers, raw_response = await self._request('GET', self.resolver.get_token_url(), headers={})
        parsed_response = self.resolver.str_to_token(raw_response)
        if parsed_response.Data is None:
            raise MalformedResponseError(f"Failed to get Token with {status=} and payload {raw_response=}")
        return status, headers, parsed_response

    async def get_request_poll(self, device_id: str) -> Tuple[int, Mapping[str, str], None]:
        status, headers, raw_response = await self._request('GET', self.resolver.get_request_poll_url(await self.token(), device_id), headers={})
        return status, headers, None

    async def get_devices_extended(self, device_id: str) -> Tuple[int, Mapping[str, str], Devices]:
        status, headers, raw_response = await self._request('GET', self.resolver.get_devices_extended_url(await self.token()), headers={})
        return status, headers, self.resolver.str_to_devices(raw_response, device_id)

    async def get_activate_scenario(self, device_id: str, scenario_id: str) -> Tuple[int, Mapping[str, str], str]:
        status, headers, raw_response = await self._request('GET', self.resolver.get_activate_scenario_url(await self.token(), device_id, scenario_id), headers={})
        return status, headers, raw_response



import json
from types import SimpleNamespace

from pyinim.cloud.types.token import Token
from pyinim.cloud.types.devices import Devices

API_CLOUD_BASEURL="https://api.inimcloud.com"

class CloudResolver:
    def __init__(self, username, password, client_id):
        self.password = password
        self.username = username
        self.client_id = client_id

    def get_token_url(self):
        data = {
            "Node": "",
            "Name": "AlienMobilePro",
            "ClientIP": "",
            "Method": "RegisterClient",
            "ClientId": "",
            "Token": "",
            "Params": {
                "Username": f'{self.username}',
                "Password": f'{self.password}',
                "ClientId": f'{self.client_id}',
                "ClientName": "Galaxy+S8+edge",
                "ClientInfo": json.dumps({
                    "name": "com.inim.alienmobile",
                    "version": "3.1.0",
                    "device": "hero2lte",
                    "brand": "samsung",
                    "platform": "android",
                    "osversion": "Oreo+v8.0,+API+Level:+26"
                }),
            "Role": "1",
            "Brand": "0"
            }
        }
        return f'{API_CLOUD_BASEURL}?req={json.dumps(data)}'

    def get_devices_extended_url(self, token):
        return f'{API_CLOUD_BASEURL}?req={{"Params":{{"Info":4223}},"Node":"","Name":"Inim Home","ClientIP":"","Method":"GetDevicesExtended","Token":"{token}","ClientId": "{self.client_id}","Context":"intrusion"}}'

    def get_request_poll_url(self, token, device_id):
        return  f'{API_CLOUD_BASEURL}?req={{"Params":{{"DeviceId":{device_id},"Type":5}},"Node":"","Name":"Inim Home","ClientIP":"","Method":"RequestPoll","Token":"{token}","ClientId":"{self.client_id}","Context":"intrusion"}}'

    def get_activate_scenario_url(self, token, device_id, scenario_id):
        return  f'{API_CLOUD_BASEURL}?req={{"Node":"","Name":"AlienMobilePro","ClientIP":"","Method":"ActivateScenario","ClientId": "{self.client_id}","Token":"{token}","Params":{{"DeviceId":"{device_id}","ScenarioId":"{scenario_id}"}}}}'
    
    def str_to_token(self, data: str):
        token: Token = json.loads(data, object_hook=lambda d: SimpleNamespace(**d))
        return token
    
    def str_to_devices(self, data: str, device_id: str):
        devices: Devices = json.loads(data, object_hook=lambda d: SimpleNamespace(**d))
        update = { device_id: getattr(devices.Data, device_id) }
        devices.Data = dict()
        devices.Data.update(**update) 

        return devices



import asyncio
import aiohttp
import time
from typing import Any, Mapping, Tuple, cast

from pyinim.cloud.resolver import CloudResolver
from pyinim.cloud.types.token import Token
from pyinim.cloud import abc

TOKEN_EXPIRATION_TIME = 86400 * 7 # 60 * 60 * 24 * 7 =  7 days < 2 months

class InimCloud(abc.InimAPI):
    def __init__(
        self, session: aiohttp.ClientSession, *args: Any, **kwargs: Any
    ) -> None:
        self._session = session
        resolver = CloudResolver(kwargs['username'], kwargs['password'], kwargs['client_id'])
        self.name = kwargs['name']
        self.expires_at = 0
        self._token = None

        # super().__init__(*args, resolver=resolver, **kwargs)
        super().__init__(self.name, resolver=resolver)

    async def _request(
        self, method: str, url: str, headers: Mapping[str, str], body: bytes = b""
    ) -> Tuple[int, Mapping[str, str], str]:
        async with self._session.request(
            method, url, headers=headers, data=body
        ) as response:
            return response.status, response.headers, await response.text() # change with await response.read() for bytes

    async def sleep(self, seconds: float) -> None:
        await asyncio.sleep(seconds)

    async def token(self) -> str:
        if not self._valid_token():
            _, _, resp = await self.get_token()
            self._token = resp.Data.Token
            self.expires_at = time.time() + TOKEN_EXPIRATION_TIME

        return self._token

    def _valid_token(self) -> bool:
        return cast(float, self.expires_at) > time.time()


import aiohttp
import asyncio
import os
from dotenv import load_dotenv

from pyinim.inim_cloud import InimCloud

load_dotenv()
INIM_USER = os.getenv('INIM_USER')
INIM_PASSWORD = os.getenv('INIM_PASSWORD')
INIM_CLIENT_ID = os.getenv('INIM_CLIENT_ID')
INIM_DEVICE_ID = os.getenv('INIM_DEVICE_ID')

def print_device_extended(devices_extended):
    print("Devices Extended: Name {}, DeviceId: {}, ActiveScenario: {}, ActiveScenarios: {}, Master: {},  Enabled: {}, NetworkStatus: {}".format(
        devices_extended.Name, devices_extended.DeviceId, devices_extended.ActiveScenario, devices_extended.ActiveScenarios, devices_extended.Enabled, devices_extended.Master, devices_extended.NetworkStatus
    ))
    for scenario in devices_extended.Scenarios:
        print("Scenario: {}, Id: {}, AreaSet: {}, AreaMask: {}, Icona: {}, Uscita: {}".format(scenario.Name, scenario.ScenarioId, scenario.AreaSet, scenario.AreaMask, scenario.Icona, scenario.Uscita))

async def poc():
    async with aiohttp.ClientSession() as session:
        inim = InimCloud(session, name="poc", username=INIM_USER, password=INIM_PASSWORD, client_id=INIM_CLIENT_ID)
        await inim.get_request_poll(INIM_DEVICE_ID)
        _st, _hs, devices_resp = await inim.get_devices_extended(INIM_DEVICE_ID)

        print_device_extended(devices_resp.Data[INIM_DEVICE_ID])

async def main():
    await poc()

asyncio.run(main())



# ------- generated by typeit -------
from typing import Any, NamedTuple, Optional, Sequence
# from typeit import TypeConstructor


class Peripherals(NamedTuple):
    Type: int
    PeripheralType: int
    ItemId: int
    Name: str
    Firmware: str
    Tamper: int
    Missing: int
    Enabled: int
    Voltage: float
    Data: str


class Zones(NamedTuple):
    ZoneId: int
    Type: int
    TerminalId: int
    Name: str
    Areas: int
    Status: int
    AlarmMemory: int
    TamperMemory: int
    Bypassed: int
    OutputOn: int
    OutputValue: int
    Power: int
    Voltage: int
    CosPhi: int
    InputType: int
    InputFlags: int
    OutputFlags: int
    Monost: int
    OutputCategory: int
    Channels: Sequence
    Favorite: int
    Special: int
    Visibility: int


class Scenarios(NamedTuple):
    ScenarioId: int
    Name: str
    AreaSet: str
    AreaMask: int
    Icona: int
    Uscita: int


class Areas(NamedTuple):
    AreaId: int
    Name: str
    Armed: int
    Alarm: int
    AlarmMemory: int
    Tamper: int
    TamperMemory: int
    AutoInsert: int


class Data(NamedTuple):
    DeviceId: int
    ActiveScenario: int
    ActiveScenarios: str
    Master: int
    Favorite: int
    CodeIndex: int
    counter_0: int
    counter_1: int
    counter_2: int
    counter_3: int
    counter_4: int
    Name: str
    InternalName: str
    SerialNumber: str
    ModelFamily: str
    ModelNumber: str
    Language: str
    ControlledBy: int
    BlockUserBinding: int
    InstReqPending: int
    Enabled: int
    EnrollDate: str
    NetworkStatus: int
    LastCommunicationDate: str
    PrimaryIpAddress: str
    PrimaryChannel: str
    SecondaryIpAddress: str
    SecondaryChannel: str
    FirmwareVersionMajor: str
    FirmwareVersionMinor: str
    TZOffset: int
    Options: int
    Faults: int
    FaultList: Sequence
    FaultEventDescription: str
    Voltage: float
    Address: str
    Lat: int
    Lng: int
    Functionalities: int
    Related: Sequence
    AlarmedAreas: str
    AlarmEventDescription: str
    Areas: Sequence[Areas]
    Scenarios: Sequence[Scenarios]
    Zones: Sequence[Zones]
    Blinds: Sequence
    Thermostats: Sequence
    Peripherals: Sequence[Peripherals]

class Devices(NamedTuple):
    Status: int
    ErrMsg: str
    ts: str
    Data: dict[str, Data]
